#include "stdafx.h"
#include "ADIS/ADIS16365.h"
#include "ADIS/ADIS16405.h"
#include "wiimote.h"


//#define WIIMOTE
#define ADIS_16365
//#define ADIS_16405


#if defined ADIS_16365
ADIS16365	*pAdis16365;
#elif defined ADIS_16405
ADIS16405	*pAdis16405;
#elif defined WIIMOTE
wiimote		*remote;
#endif


#if defined WIIMOTE
void on_state_change (wiimote &remote, state_change_flags changed)
{
	// a MotionPlus was detected
	if(changed & MOTIONPLUS_DETECTED) {
		// enable it if there isn't a normal extension plugged into it
		// (MotionPlus devices don't report like normal extensions until
		//  enabled - and then, other extensions attached to it will no longer be
		//  reported (so disable it when you want to access to those again).
		if(remote.ExtensionType == wiimote_state::NONE) {
			bool res = remote.EnableMotionPlus();
			_ASSERT(res);
		}
	}
	else if(changed & MOTIONPLUS_EXTENSION_CONNECTED) {
		// an extension is connected to the MotionPlus.  We can't read it if the
		//  MotionPlus is currently enabled, so disable it:
		if(remote.MotionPlusEnabled())
			remote.DisableMotionPlus();
	}
	else if(changed & MOTIONPLUS_EXTENSION_DISCONNECTED) {
		// the extension plugged into the MotionPlus was removed, so enable
		//  the MotionPlus data again:
		if(remote.MotionPlusConnected())
			remote.EnableMotionPlus();
	}
	// extension was just connected:
	else if(changed & EXTENSION_CONNECTED) {
		// switch to a report mode that includes the extension data (we will
		//  loose the IR dot sizes)
		// note: there is no need to set report types for a Balance Board.
		if(!remote.IsBalanceBoard())
			remote.SetReportType(wiimote::IN_BUTTONS_ACCEL_IR_EXT);
	}
	// extension was just disconnected:
	else if(changed & EXTENSION_DISCONNECTED) {
		// use a non-extension report mode (this gives us back the IR dot sizes)
		remote.SetReportType(wiimote::IN_BUTTONS_ACCEL_IR);
	}
}
#endif

void InitDevice()
{
#if defined ADIS_16365
	pAdis16365 = new ADIS16365();
#elif defined ADIS_16405
	pAdis16405 = new ADIS16405();
#elif defined WIIMOTE
	remote = new wiimote();
	remote->ChangedCallback		= on_state_change;
	remote->CallbackTriggerFlags = (state_change_flags) (EXTENSION_CHANGED | MOTIONPLUS_CHANGED);
#endif
}

void DeinitDevice()
{
#if defined ADIS_16365
	delete pAdis16365;
#elif defined ADIS_16405
	delete pAdis16405;
#elif defined WIIMOTE
	delete remote;
#endif
}

void ConnectDevice()
{
#if defined WIIMOTE
	remote->Connect(wiimote::FIRST_AVAILABLE);

	// connected - light all LEDs
	remote->SetLEDs(0x0f);

	remote->SetReportType (wiimote::IN_BUTTONS_ACCEL_IR_EXT);
#endif
}

void ReadDevice(double gyro[3], double accl[3], double magn[3], double temp[3])
{
#if defined ADIS_16365
	double data[ADIS16365_DATA_NUM] = {0., };
	pAdis16365->GetMeasure(data);

	gyro[0] = data[1];
	gyro[1] = data[2];
	gyro[2] = data[3];
	
	accl[0] = data[4];
	accl[1] = data[5];
	accl[2] = data[6];
	
	magn[0] = 0.;
	magn[1] = 0.;
	magn[2] = 0.;
	
	temp[0] = data[7];
	temp[1] = data[8];
	temp[2] = data[9];
	
#elif defined ADIS_16405
	double data[ADIS16405_DATA_NUM] = {0., };
	pAdis16405->GetMeasure(data);

	gyro[0] = data[1];
	gyro[1] = data[2];
	gyro[2] = data[3];
	
	accl[0] = data[4];
	accl[1] = data[5];
	accl[2] = data[6];
	
	magn[0] = data[7];
	magn[1] = data[8];
	magn[2] = data[9];
	
	temp[0] = data[10];
	temp[1] = data[10];
	temp[2] = data[10];
	

#elif defined WIIMOTE
	static unsigned lit_led          = 0;	

	remote->SetLEDs((BYTE)(1<<lit_led));
	lit_led		  = (++lit_led) % 4;

	remote->RefreshState();

	// did we loose the connection?
	//	if(remote->ConnectionLost()) return;

	gyro[0] = remote->MotionPlus.Speed.Roll;
	gyro[1] = -remote->MotionPlus.Speed.Pitch;
	gyro[2] = -remote->MotionPlus.Speed.Yaw;
	
	accl[0] = remote->Acceleration.Y;
	accl[1] = -remote->Acceleration.X;
	accl[2] = -remote->Acceleration.Z;
	
	magn[0] = 0.;
	magn[1] = 0.;
	magn[2] = 0.;
	
	temp[0] = 0.;
	temp[1] = 0.;
	temp[2] = 0.;

#endif
}